unit Consoles;

{ TConsole component (C) 1999, by Paul TOTH <tothpaul@multimania.com>
  http://www.multimania.com/tothpaul

  Properties :

   - Rows:integer;
   - Cols:integer;
   - Font:TRomFont;
   - Scroll:boolean; (on last line)

  Provide a Text Console with BP7 like function :

   - function TextColor(0..15);
   - function TextBackGround(0..15);
   - TextAttr:word; (16*BackGround+Color)
   - procedure GotoXY(x,y:integer);
   - Procedure Write(s:string);
   - Procedure WriteLn(s:string);
   - Function WhereX:integer;
   - Function WhereY:integer;
   - Procedure ClrScr;
   - Function ClrEOL;
   - Mem[Index:integer]:byte;  like Mem[SegB800:ofs]
   - MemW[Index:integer]:word; like MemW[SegB800:ofs]

  and :

   - PutPixel(x,y,color:integer);
   - Line(x1,y1,x2,y2,color:integer);
   - Ellipse(x1,y1,x2,y2,color:integer);

}

{
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

Const
 Black         =0;
 Blue          =1;
 Green         =2;
 Cyan          =3;
 Red           =4;
 Magenta       =5;
 Brown         =6;
 LightGray     =7;
 DarkGray      =8;
 LightBlue     =9;
 LightGreen    =10;
 LightCyan     =11;
 LightRed      =12;
 LightMagenta  =13;
 Yellow        =14;
 White         =15;

 CO40=1;
 CO80=3;
 Font8x8=256;

type
  TRomFont=(Fnt8x8,Fnt8x16);

{ Replace the TBitmapInfo for dib_pal_colors on StretchDIBits API }
 TPalBitmapInfo=record
  bmiHeader:TBitmapInfoHeader;
  bmiColors:Array[0..15] of Word;
 end;

  TConsole = class(TCustomControl)
  private
   fCols:integer;
   fRows:integer;
// fAutoSize:boolean;
   fFont:TRomFont;
   fFontHeight:integer;
   fMem :pchar;
   fMemSize:integer;
   fWhereX:integer;
   fWhereY:integer;
   fPalette:HPalette;
   fDIB:pchar; // Bitmap DIB
   fDibWidth:integer;
   fDibHeight:integer;
   fBitmapInfo:TPalBitmapInfo;
   fTextAttr:byte;
   fOEMConvert:boolean;
   fUpdate:integer;
   fScroll:boolean;
   Procedure SetCols(Value:integer);
   Procedure SetRows(Value:integer);
//   Procedure SetAutoSize(Value:boolean);
   Function GetMemB(Index:integer):byte;
   Procedure SetMemB(Index:integer; Value:byte);
   Function GetMemW(Index:integer):word;
   Procedure SetMemW(Index:integer; Value:word);
   Procedure SetFont(Value:TRomFont);
   procedure WMSetFocus(var Message: TWMSetFocus); message WM_SETFOCUS;
   procedure WMKillFocus(Var Message: TWMKillFocus); message WM_KILLFOCUS;
   procedure WMGetDlgCode(Var Msg:TWMGetDlgCode); message wm_getdlgcode;
   procedure WMEraseBkGnd(Var Msg:TMessage); message wm_erasebkgnd;
  protected
   function CreatePalette16:HPalette;
   procedure DoAutoSize;
   procedure Newdib;
   procedure PutChar(x,y:integer; c:char; color:byte);
   procedure SetChar(x,y:integer; c:char; color:byte);
   procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
   procedure KeyDown(var Key: Word; Shift: TShiftState); override;
   procedure KeyPress(var Key: Char); override;
   procedure ScrollUp;
   procedure MoveTo(x,y:integer);
  public
   Constructor Create(AOwner:TComponent); override;
   Destructor Destroy; override;
   Procedure Loaded; override;
   Procedure Paint; override;
   Property Mem[Index:integer]:byte read GetMemB write SetMemB;
   Property MemW[Index:integer]:word read GetMemW write SetMemW;
   Procedure BeginUpdate;
   Procedure EndUpdate;
   procedure ClrScr;
   procedure ClrEOL;
   procedure TextMode(Mode:Integer);
   Function WhereX:integer;
   Function WhereY:integer;
   Procedure GotoXY(X,Y:integer);
   Procedure Write(s:string);
   Procedure WriteLn(s:string);
   Procedure PutPixel(x,y,c:integer);
   Property TextAttr:byte read fTextAttr write fTextAttr;
   Procedure TextColor(color:byte);
   Procedure TextBackGround(Color:byte);
   Procedure Line(x1,y1,x2,y2,color:integer);
   Procedure Ellipse(x1,y1,x2,y2,color:integer);
  published
   Property Cols:integer read fCols write SetCols default 80;
   Property Rows:integer read fRows write SetRows default 25;
// Property AutoSize:boolean read fAutoSize write SetAutoSize default false;
   Property Font:TRomFont read fFont write SetFont default Fnt8x16;
   property OEMConvert:boolean read fOEMConvert write fOEMConvert default true;
   Property Scroll:boolean read fScroll write fScroll default true;
   Property Enabled;
   Property TabOrder;
   Property TabStop default true;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('MySoft', [TConsole]);
end;

Type
 TRom8x8=array[#0..#255,0..7] of byte;
 TRom8x16=array[#0..#255,0..15] of byte;
 TDosPalette=array[0..15] of TPaletteEntry;

Const
 Rom8x8:TRom8x8=(
 {#  0} (  0,  0,  0,  0,  0,  0,  0,  0),
 {#  1} (126,129,165,129,189,153,129,126),
 {#  2} (126,255,219,255,195,231,255,126),
 {#  3} (108,254,254,254,124, 56, 16,  0),
 {#  4} ( 16, 56,124,254,124, 56, 16,  0),
 {#  5} ( 56,124, 56,254,254,124, 56,124),
 {#  6} ( 16, 16, 56,124,254,124, 56,124),
 {#  7} (  0,  0, 24, 60, 60, 24,  0,  0),
 {#  8} (255,255,231,195,195,231,255,255),
 {#  9} (  0, 60,102, 66, 66,102, 60,  0),
 {# 10} (255,195,153,189,189,153,195,255),
 {# 11} ( 15,  7, 15,125,204,204,204,120),
 {# 12} ( 60,102,102,102, 60, 24,126, 24),
 {# 13} ( 63, 51, 63, 48, 48,112,240,224),
 {# 14} (127, 99,127, 99, 99,103,230,192),
 {# 15} (153, 90, 60,231,231, 60, 90,153),
 {# 16} (128,224,248,254,248,224,128,  0),
 {# 17} (  2, 14, 62,254, 62, 14,  2,  0),
 {# 18} ( 24, 60,126, 24, 24,126, 60, 24),
 {# 19} (102,102,102,102,102,  0,102,  0),
 {# 20} (127,219,219,123, 27, 27, 27,  0),
 {# 21} ( 62, 99, 56,108,108, 56,204,120),
 {# 22} (  0,  0,  0,  0,126,126,126,  0),
 {# 23} ( 24, 60,126, 24,126, 60, 24,255),
 {# 24} ( 24, 60,126, 24, 24, 24, 24,  0),
 {# 25} ( 24, 24, 24, 24,126, 60, 24,  0),
 {# 26} (  0, 24, 12,254, 12, 24,  0,  0),
 {# 27} (  0, 48, 96,254, 96, 48,  0,  0),
 {# 28} (  0,  0,192,192,192,254,  0,  0),
 {# 29} (  0, 36,102,255,102, 36,  0,  0),
 {# 30} (  0, 24, 60,126,255,255,  0,  0),
 {# 31} (  0,255,255,126, 60, 24,  0,  0),
 {# 32} (  0,  0,  0,  0,  0,  0,  0,  0),
 {# 33} ( 48,120,120, 48, 48,  0, 48,  0),
 {# 34} (108,108,108,  0,  0,  0,  0,  0),
 {# 35} (108,108,254,108,254,108,108,  0),
 {# 36} ( 48,124,192,120, 12,248, 48,  0),
 {# 37} (  0,198,204, 24, 48,102,198,  0),
 {# 38} ( 56,108, 56,118,220,204,118,  0),
 {# 39} ( 96, 96,192,  0,  0,  0,  0,  0),
 {# 40} ( 24, 48, 96, 96, 96, 48, 24,  0),
 {# 41} ( 96, 48, 24, 24, 24, 48, 96,  0),
 {# 42} (  0,102, 60,255, 60,102,  0,  0),
 {# 43} (  0, 48, 48,252, 48, 48,  0,  0),
 {# 44} (  0,  0,  0,  0,  0, 48, 48, 96),
 {# 45} (  0,  0,  0,252,  0,  0,  0,  0),
 {# 46} (  0,  0,  0,  0,  0, 48, 48,  0),
 {# 47} (  6, 12, 24, 48, 96,192,128,  0),
 {# 48} (124,198,206,222,246,230,124,  0),
 {# 49} ( 48,112, 48, 48, 48, 48,252,  0),
 {# 50} (120,204, 12, 56, 96,204,252,  0),
 {# 51} (120,204, 12, 56, 12,204,120,  0),
 {# 52} ( 28, 60,108,204,254, 12, 30,  0),
 {# 53} (252,192,248, 12, 12,204,120,  0),
 {# 54} ( 56, 96,192,248,204,204,120,  0),
 {# 55} (252,204, 12, 24, 48, 48, 48,  0),
 {# 56} (120,204,204,120,204,204,120,  0),
 {# 57} (120,204,204,124, 12, 24,112,  0),
 {# 58} (  0, 48, 48,  0,  0, 48, 48,  0),
 {# 59} (  0, 48, 48,  0,  0, 48, 48, 96),
 {# 60} ( 24, 48, 96,192, 96, 48, 24,  0),
 {# 61} (  0,  0,252,  0,  0,252,  0,  0),
 {# 62} ( 96, 48, 24, 12, 24, 48, 96,  0),
 {# 63} (120,204, 12, 24, 48,  0, 48,  0),
 {# 64} (124,198,222,222,222,192,120,  0),
 {# 65} ( 48,120,204,204,252,204,204,  0),
 {# 66} (252,102,102,124,102,102,252,  0),
 {# 67} ( 60,102,192,192,192,102, 60,  0),
 {# 68} (248,108,102,102,102,108,248,  0),
 {# 69} (254, 98,104,120,104, 98,254,  0),
 {# 70} (254, 98,104,120,104, 96,240,  0),
 {# 71} ( 60,102,192,192,206,102, 62,  0),
 {# 72} (204,204,204,252,204,204,204,  0),
 {# 73} (120, 48, 48, 48, 48, 48,120,  0),
 {# 74} ( 30, 12, 12, 12,204,204,120,  0),
 {# 75} (230,102,108,120,108,102,230,  0),
 {# 76} (240, 96, 96, 96, 98,102,254,  0),
 {# 77} (198,238,254,254,214,198,198,  0),
 {# 78} (198,230,246,222,206,198,198,  0),
 {# 79} ( 56,108,198,198,198,108, 56,  0),
 {# 80} (252,102,102,124, 96, 96,240,  0),
 {# 81} (120,204,204,204,220,120, 28,  0),
 {# 82} (252,102,102,124,108,102,230,  0),
 {# 83} (120,204,224,112, 28,204,120,  0),
 {# 84} (252,180, 48, 48, 48, 48,120,  0),
 {# 85} (204,204,204,204,204,204,252,  0),
 {# 86} (204,204,204,204,204,120, 48,  0),
 {# 87} (198,198,198,214,254,238,198,  0),
 {# 88} (198,198,108, 56, 56,108,198,  0),
 {# 89} (204,204,204,120, 48, 48,120,  0),
 {# 90} (254,198,140, 24, 50,102,254,  0),
 {# 91} (120, 96, 96, 96, 96, 96,120,  0),
 {# 92} (192, 96, 48, 24, 12,  6,  2,  0),
 {# 93} (120, 24, 24, 24, 24, 24,120,  0),
 {# 94} ( 16, 56,108,198,  0,  0,  0,  0),
 {# 95} (  0,  0,  0,  0,  0,  0,  0,255),
 {# 96} ( 48, 48, 24,  0,  0,  0,  0,  0),
 {# 97} (  0,  0,120, 12,124,204,118,  0),
 {# 98} (224, 96, 96,124,102,102,220,  0),
 {# 99} (  0,  0,120,204,192,204,120,  0),
 {#100} ( 28, 12, 12,124,204,204,118,  0),
 {#101} (  0,  0,120,204,252,192,120,  0),
 {#102} ( 56,108, 96,240, 96, 96,240,  0),
 {#103} (  0,  0,118,204,204,124, 12,248),
 {#104} (224, 96,108,118,102,102,230,  0),
 {#105} ( 48,  0,112, 48, 48, 48,120,  0),
 {#106} ( 12,  0, 12, 12, 12,204,204,120),
 {#107} (224, 96,102,108,120,108,230,  0),
 {#108} (112, 48, 48, 48, 48, 48,120,  0),
 {#109} (  0,  0,204,254,254,214,198,  0),
 {#110} (  0,  0,248,204,204,204,204,  0),
 {#111} (  0,  0,120,204,204,204,120,  0),
 {#112} (  0,  0,220,102,102,124, 96,240),
 {#113} (  0,  0,118,204,204,124, 12, 30),
 {#114} (  0,  0,220,118,102, 96,240,  0),
 {#115} (  0,  0,124,192,120, 12,248,  0),
 {#116} ( 16, 48,124, 48, 48, 52, 24,  0),
 {#117} (  0,  0,204,204,204,204,118,  0),
 {#118} (  0,  0,204,204,204,120, 48,  0),
 {#119} (  0,  0,198,214,254,254,108,  0),
 {#120} (  0,  0,198,108, 56,108,198,  0),
 {#121} (  0,  0,204,204,204,124, 12,248),
 {#122} (  0,  0,252,152, 48,100,252,  0),
 {#123} ( 28, 48, 48,224, 48, 48, 28,  0),
 {#124} ( 24, 24, 24,  0, 24, 24, 24,  0),
 {#125} (224, 48, 48, 28, 48, 48,224,  0),
 {#126} (118,220,  0,  0,  0,  0,  0,  0),
 {#127} (  0, 16, 56,108,198,198,254,  0),
 {#128} (120,204,192,204,120, 24, 12,120),
 {#129} (  0,204,  0,204,204,204,126,  0),
 {#130} ( 28,  0,120,204,252,192,120,  0),
 {#131} (126,195, 60,  6, 62,102, 63,  0),
 {#132} (204,  0,120, 12,124,204,126,  0),
 {#133} (224,  0,120, 12,124,204,126,  0),
 {#134} ( 48, 48,120, 12,124,204,126,  0),
 {#135} (  0,  0,120,192,192,120, 12, 56),
 {#136} (126,195, 60,102,126, 96, 60,  0),
 {#137} (204,  0,120,204,252,192,120,  0),
 {#138} (224,  0,120,204,252,192,120,  0),
 {#139} (204,  0,112, 48, 48, 48,120,  0),
 {#140} (124,198, 56, 24, 24, 24, 60,  0),
 {#141} (224,  0,112, 48, 48, 48,120,  0),
 {#142} (198, 56,108,198,254,198,198,  0),
 {#143} ( 48, 48,  0,120,204,252,204,  0),
 {#144} ( 28,  0,252, 96,120, 96,252,  0),
 {#145} (  0,  0,127, 12,127,204,127,  0),
 {#146} ( 62,108,204,254,204,204,206,  0),
 {#147} (120,204,  0,120,204,204,120,  0),
 {#148} (  0,204,  0,120,204,204,120,  0),
 {#149} (  0,224,  0,120,204,204,120,  0),
 {#150} (120,204,  0,204,204,204,126,  0),
 {#151} (  0,224,  0,204,204,204,126,  0),
 {#152} (  0,204,  0,204,204,124, 12,248),
 {#153} (195, 24, 60,102,102, 60, 24,  0),
 {#154} (204,  0,204,204,204,204,120,  0),
 {#155} ( 24, 24,126,192,192,126, 24, 24),
 {#156} ( 56,108,100,240, 96,230,252,  0),
 {#157} (204,204,120,252, 48,252, 48, 48),
 {#158} (248,204,204,250,198,207,198,199),
 {#159} ( 14, 27, 24, 60, 24, 24,216,112),
 {#160} ( 28,  0,120, 12,124,204,126,  0),
 {#161} ( 56,  0,112, 48, 48, 48,120,  0),
 {#162} (  0, 28,  0,120,204,204,120,  0),
 {#163} (  0, 28,  0,204,204,204,126,  0),
 {#164} (  0,248,  0,248,204,204,204,  0),
 {#165} (252,  0,204,236,252,220,204,  0),
 {#166} ( 60,108,108, 62,  0,126,  0,  0),
 {#167} ( 56,108,108, 56,  0,124,  0,  0),
 {#168} ( 48,  0, 48, 96,192,204,120,  0),
 {#169} (  0,  0,  0,252,192,192,  0,  0),
 {#170} (  0,  0,  0,252, 12, 12,  0,  0),
 {#171} (195,198,204,222, 51,102,204, 15),
 {#172} (195,198,204,219, 55,111,207,  3),
 {#173} ( 24, 24,  0, 24, 24, 24, 24,  0),
 {#174} (  0, 51,102,204,102, 51,  0,  0),
 {#175} (  0,204,102, 51,102,204,  0,  0),
 {#176} ( 34,136, 34,136, 34,136, 34,136),
 {#177} ( 85,170, 85,170, 85,170, 85,170),
 {#178} (219,119,219,238,219,119,219,238),
 {#179} ( 24, 24, 24, 24, 24, 24, 24, 24),
 {#180} ( 24, 24, 24, 24,248, 24, 24, 24),
 {#181} ( 24, 24,248, 24,248, 24, 24, 24),
 {#182} ( 54, 54, 54, 54,246, 54, 54, 54),
 {#183} (  0,  0,  0,  0,254, 54, 54, 54),
 {#184} (  0,  0,248, 24,248, 24, 24, 24),
 {#185} ( 54, 54,246,  6,246, 54, 54, 54),
 {#186} ( 54, 54, 54, 54, 54, 54, 54, 54),
 {#187} (  0,  0,254,  6,246, 54, 54, 54),
 {#188} ( 54, 54,246,  6,254,  0,  0,  0),
 {#189} ( 54, 54, 54, 54,254,  0,  0,  0),
 {#190} ( 24, 24,248, 24,248,  0,  0,  0),
 {#191} (  0,  0,  0,  0,248, 24, 24, 24),
 {#192} ( 24, 24, 24, 24, 31,  0,  0,  0),
 {#193} ( 24, 24, 24, 24,255,  0,  0,  0),
 {#194} (  0,  0,  0,  0,255, 24, 24, 24),
 {#195} ( 24, 24, 24, 24, 31, 24, 24, 24),
 {#196} (  0,  0,  0,  0,255,  0,  0,  0),
 {#197} ( 24, 24, 24, 24,255, 24, 24, 24),
 {#198} ( 24, 24, 31, 24, 31, 24, 24, 24),
 {#199} ( 54, 54, 54, 54, 55, 54, 54, 54),
 {#200} ( 54, 54, 55, 48, 63,  0,  0,  0),
 {#201} (  0,  0, 63, 48, 55, 54, 54, 54),
 {#202} ( 54, 54,247,  0,255,  0,  0,  0),
 {#203} (  0,  0,255,  0,247, 54, 54, 54),
 {#204} ( 54, 54, 55, 48, 55, 54, 54, 54),
 {#205} (  0,  0,255,  0,255,  0,  0,  0),
 {#206} ( 54, 54,247,  0,247, 54, 54, 54),
 {#207} ( 24, 24,255,  0,255,  0,  0,  0),
 {#208} ( 54, 54, 54, 54,255,  0,  0,  0),
 {#209} (  0,  0,255,  0,255, 24, 24, 24),
 {#210} (  0,  0,  0,  0,255, 54, 54, 54),
 {#211} ( 54, 54, 54, 54, 63,  0,  0,  0),
 {#212} ( 24, 24, 31, 24, 31,  0,  0,  0),
 {#213} (  0,  0, 31, 24, 31, 24, 24, 24),
 {#214} (  0,  0,  0,  0, 63, 54, 54, 54),
 {#215} ( 54, 54, 54, 54,255, 54, 54, 54),
 {#216} ( 24, 24,255, 24,255, 24, 24, 24),
 {#217} ( 24, 24, 24, 24,248,  0,  0,  0),
 {#218} (  0,  0,  0,  0, 31, 24, 24, 24),
 {#219} (255,255,255,255,255,255,255,255),
 {#220} (  0,  0,  0,  0,255,255,255,255),
 {#221} (240,240,240,240,240,240,240,240),
 {#222} ( 15, 15, 15, 15, 15, 15, 15, 15),
 {#223} (255,255,255,255,  0,  0,  0,  0),
 {#224} (  0,  0,118,220,200,220,118,  0),
 {#225} (  0,120,204,248,204,248,192,192),
 {#226} (  0,252,204,192,192,192,192,  0),
 {#227} (  0,254,108,108,108,108,108,  0),
 {#228} (252,204, 96, 48, 96,204,252,  0),
 {#229} (  0,  0,126,216,216,216,112,  0),
 {#230} (  0,102,102,102,102,124, 96,192),
 {#231} (  0,118,220, 24, 24, 24, 24,  0),
 {#232} (252, 48,120,204,204,120, 48,252),
 {#233} ( 56,108,198,254,198,108, 56,  0),
 {#234} ( 56,108,198,198,108,108,238,  0),
 {#235} ( 28, 48, 24,124,204,204,120,  0),
 {#236} (  0,  0,126,219,219,126,  0,  0),
 {#237} (  6, 12,126,219,219,126, 96,192),
 {#238} ( 56, 96,192,248,192, 96, 56,  0),
 {#239} (120,204,204,204,204,204,204,  0),
 {#240} (  0,252,  0,252,  0,252,  0,  0),
 {#241} ( 48, 48,252, 48, 48,  0,252,  0),
 {#242} ( 96, 48, 24, 48, 96,  0,252,  0),
 {#243} ( 24, 48, 96, 48, 24,  0,252,  0),
 {#244} ( 14, 27, 27, 24, 24, 24, 24, 24),
 {#245} ( 24, 24, 24, 24, 24,216,216,112),
 {#246} ( 48, 48,  0,252,  0, 48, 48,  0),
 {#247} (  0,118,220,  0,118,220,  0,  0),
 {#248} ( 56,108,108, 56,  0,  0,  0,  0),
 {#249} (  0,  0,  0, 24, 24,  0,  0,  0),
 {#250} (  0,  0,  0,  0, 24,  0,  0,  0),
 {#251} ( 15, 12, 12, 12,236,108, 60, 28),
 {#252} (120,108,108,108,108,  0,  0,  0),
 {#253} (112, 24, 48, 96,120,  0,  0,  0),
 {#254} (  0,  0, 60, 60, 60, 60,  0,  0),
 {#255} (  0,  0,  0,  0,  0,  0,  0,  0)
 );
 Rom8x16:TRom8x16=(
 {#  0} (  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#  1} (  0,  0,126,129,165,129,129,189,153,129,129,126,  0,  0,  0,  0),
 {#  2} (  0,  0,126,255,219,255,255,195,231,255,255,126,  0,  0,  0,  0),
 {#  3} (  0,  0,  0,  0,108,254,254,254,254,124, 56, 16,  0,  0,  0,  0),
 {#  4} (  0,  0,  0,  0, 16, 56,124,254,124, 56, 16,  0,  0,  0,  0,  0),
 {#  5} (  0,  0,  0, 24, 60, 60,231,231,231, 24, 24, 60,  0,  0,  0,  0),
 {#  6} (  0,  0,  0, 24, 60,126,255,255,126, 24, 24, 60,  0,  0,  0,  0),
 {#  7} (  0,  0,  0,  0,  0,  0, 24, 60, 60, 24,  0,  0,  0,  0,  0,  0),
 {#  8} (255,255,255,255,255,255,231,195,195,231,255,255,255,255,255,255),
 {#  9} (  0,  0,  0,  0,  0, 60,102, 66, 66,102, 60,  0,  0,  0,  0,  0),
 {# 10} (255,255,255,255,255,195,153,189,189,153,195,255,255,255,255,255),
 {# 11} (  0,  0, 30, 14, 26, 50,120,204,204,204,204,120,  0,  0,  0,  0),
 {# 12} (  0,  0, 60,102,102,102,102, 60, 24,126, 24, 24,  0,  0,  0,  0),
 {# 13} (  0,  0, 63, 51, 63, 48, 48, 48, 48,112,240,224,  0,  0,  0,  0),
 {# 14} (  0,  0,127, 99,127, 99, 99, 99, 99,103,231,230,192,  0,  0,  0),
 {# 15} (  0,  0,  0, 24, 24,219, 60,231, 60,219, 24, 24,  0,  0,  0,  0),
 {# 16} (  0,128,192,224,240,248,254,248,240,224,192,128,  0,  0,  0,  0),
 {# 17} (  0,  2,  6, 14, 30, 62,254, 62, 30, 14,  6,  2,  0,  0,  0,  0),
 {# 18} (  0,  0, 24, 60,126, 24, 24, 24,126, 60, 24,  0,  0,  0,  0,  0),
 {# 19} (  0,  0,102,102,102,102,102,102,102,  0,102,102,  0,  0,  0,  0),
 {# 20} (  0,  0,127,219,219,219,123, 27, 27, 27, 27, 27,  0,  0,  0,  0),
 {# 21} (  0,124,198, 96, 56,108,198,198,108, 56, 12,198,124,  0,  0,  0),
 {# 22} (  0,  0,  0,  0,  0,  0,  0,  0,254,254,254,254,  0,  0,  0,  0),
 {# 23} (  0,  0, 24, 60,126, 24, 24, 24,126, 60, 24,126,  0,  0,  0,  0),
 {# 24} (  0,  0, 24, 60,126, 24, 24, 24, 24, 24, 24, 24,  0,  0,  0,  0),
 {# 25} (  0,  0, 24, 24, 24, 24, 24, 24, 24,126, 60, 24,  0,  0,  0,  0),
 {# 26} (  0,  0,  0,  0,  0, 24, 12,254, 12, 24,  0,  0,  0,  0,  0,  0),
 {# 27} (  0,  0,  0,  0,  0, 48, 96,254, 96, 48,  0,  0,  0,  0,  0,  0),
 {# 28} (  0,  0,  0,  0,  0,  0,192,192,192,254,  0,  0,  0,  0,  0,  0),
 {# 29} (  0,  0,  0,  0,  0, 40,108,254,108, 40,  0,  0,  0,  0,  0,  0),
 {# 30} (  0,  0,  0,  0, 16, 56, 56,124,124,254,254,  0,  0,  0,  0,  0),
 {# 31} (  0,  0,  0,  0,254,254,124,124, 56, 56, 16,  0,  0,  0,  0,  0),
 {# 32} (  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {# 33} (  0,  0, 24, 60, 60, 60, 24, 24, 24,  0, 24, 24,  0,  0,  0,  0),
 {# 34} (  0,102,102,102, 36,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {# 35} (  0,  0,  0,108,108,254,108,108,108,254,108,108,  0,  0,  0,  0),
 {# 36} ( 24, 24,124,198,194,192,124,  6,  6,134,198,124, 24, 24,  0,  0),
 {# 37} (  0,  0,  0,  0,194,198, 12, 24, 48, 96,198,134,  0,  0,  0,  0),
 {# 38} (  0,  0, 56,108,108, 56,118,220,204,204,204,118,  0,  0,  0,  0),
 {# 39} (  0, 48, 48, 48, 96,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {# 40} (  0,  0, 12, 24, 48, 48, 48, 48, 48, 48, 24, 12,  0,  0,  0,  0),
 {# 41} (  0,  0, 48, 24, 12, 12, 12, 12, 12, 12, 24, 48,  0,  0,  0,  0),
 {# 42} (  0,  0,  0,  0,  0,102, 60,255, 60,102,  0,  0,  0,  0,  0,  0),
 {# 43} (  0,  0,  0,  0,  0, 24, 24,126, 24, 24,  0,  0,  0,  0,  0,  0),
 {# 44} (  0,  0,  0,  0,  0,  0,  0,  0,  0, 24, 24, 24, 48,  0,  0,  0),
 {# 45} (  0,  0,  0,  0,  0,  0,  0,254,  0,  0,  0,  0,  0,  0,  0,  0),
 {# 46} (  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 24, 24,  0,  0,  0,  0),
 {# 47} (  0,  0,  0,  0,  2,  6, 12, 24, 48, 96,192,128,  0,  0,  0,  0),
 {# 48} (  0,  0, 56,108,198,198,214,214,198,198,108, 56,  0,  0,  0,  0),
 {# 49} (  0,  0, 24, 56,120, 24, 24, 24, 24, 24, 24,126,  0,  0,  0,  0),
 {# 50} (  0,  0,124,198,  6, 12, 24, 48, 96,192,198,254,  0,  0,  0,  0),
 {# 51} (  0,  0,124,198,  6,  6, 60,  6,  6,  6,198,124,  0,  0,  0,  0),
 {# 52} (  0,  0, 12, 28, 60,108,204,254, 12, 12, 12, 30,  0,  0,  0,  0),
 {# 53} (  0,  0,254,192,192,192,252,  6,  6,  6,198,124,  0,  0,  0,  0),
 {# 54} (  0,  0, 56, 96,192,192,252,198,198,198,198,124,  0,  0,  0,  0),
 {# 55} (  0,  0,254,198,  6,  6, 12, 24, 48, 48, 48, 48,  0,  0,  0,  0),
 {# 56} (  0,  0,124,198,198,198,124,198,198,198,198,124,  0,  0,  0,  0),
 {# 57} (  0,  0,124,198,198,198,126,  6,  6,  6, 12,120,  0,  0,  0,  0),
 {# 58} (  0,  0,  0,  0, 24, 24,  0,  0,  0, 24, 24,  0,  0,  0,  0,  0),
 {# 59} (  0,  0,  0,  0, 24, 24,  0,  0,  0, 24, 24, 48,  0,  0,  0,  0),
 {# 60} (  0,  0,  0,  6, 12, 24, 48, 96, 48, 24, 12,  6,  0,  0,  0,  0),
 {# 61} (  0,  0,  0,  0,  0,126,  0,  0,126,  0,  0,  0,  0,  0,  0,  0),
 {# 62} (  0,  0,  0, 96, 48, 24, 12,  6, 12, 24, 48, 96,  0,  0,  0,  0),
 {# 63} (  0,  0,124,198,198, 12, 24, 24, 24,  0, 24, 24,  0,  0,  0,  0),
 {# 64} (  0,  0,  0,124,198,198,222,222,222,220,192,124,  0,  0,  0,  0),
 {# 65} (  0,  0, 16, 56,108,198,198,254,198,198,198,198,  0,  0,  0,  0),
 {# 66} (  0,  0,252,102,102,102,124,102,102,102,102,252,  0,  0,  0,  0),
 {# 67} (  0,  0, 60,102,194,192,192,192,192,194,102, 60,  0,  0,  0,  0),
 {# 68} (  0,  0,248,108,102,102,102,102,102,102,108,248,  0,  0,  0,  0),
 {# 69} (  0,  0,254,102, 98,104,120,104, 96, 98,102,254,  0,  0,  0,  0),
 {# 70} (  0,  0,254,102, 98,104,120,104, 96, 96, 96,240,  0,  0,  0,  0),
 {# 71} (  0,  0, 60,102,194,192,192,222,198,198,102, 58,  0,  0,  0,  0),
 {# 72} (  0,  0,198,198,198,198,254,198,198,198,198,198,  0,  0,  0,  0),
 {# 73} (  0,  0, 60, 24, 24, 24, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {# 74} (  0,  0, 30, 12, 12, 12, 12, 12,204,204,204,120,  0,  0,  0,  0),
 {# 75} (  0,  0,230,102,102,108,120,120,108,102,102,230,  0,  0,  0,  0),
 {# 76} (  0,  0,240, 96, 96, 96, 96, 96, 96, 98,102,254,  0,  0,  0,  0),
 {# 77} (  0,  0,198,238,254,254,214,198,198,198,198,198,  0,  0,  0,  0),
 {# 78} (  0,  0,198,230,246,254,222,206,198,198,198,198,  0,  0,  0,  0),
 {# 79} (  0,  0,124,198,198,198,198,198,198,198,198,124,  0,  0,  0,  0),
 {# 80} (  0,  0,252,102,102,102,124, 96, 96, 96, 96,240,  0,  0,  0,  0),
 {# 81} (  0,  0,124,198,198,198,198,198,198,214,222,124, 12, 14,  0,  0),
 {# 82} (  0,  0,252,102,102,102,124,108,102,102,102,230,  0,  0,  0,  0),
 {# 83} (  0,  0,124,198,198, 96, 56, 12,  6,198,198,124,  0,  0,  0,  0),
 {# 84} (  0,  0,126,126, 90, 24, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {# 85} (  0,  0,198,198,198,198,198,198,198,198,198,124,  0,  0,  0,  0),
 {# 86} (  0,  0,198,198,198,198,198,198,198,108, 56, 16,  0,  0,  0,  0),
 {# 87} (  0,  0,198,198,198,198,214,214,214,254,238,108,  0,  0,  0,  0),
 {# 88} (  0,  0,198,198,108,124, 56, 56,124,108,198,198,  0,  0,  0,  0),
 {# 89} (  0,  0,102,102,102,102, 60, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {# 90} (  0,  0,254,198,134, 12, 24, 48, 96,194,198,254,  0,  0,  0,  0),
 {# 91} (  0,  0, 60, 48, 48, 48, 48, 48, 48, 48, 48, 60,  0,  0,  0,  0),
 {# 92} (  0,  0,  0,128,192,224,112, 56, 28, 14,  6,  2,  0,  0,  0,  0),
 {# 93} (  0,  0, 60, 12, 12, 12, 12, 12, 12, 12, 12, 60,  0,  0,  0,  0),
 {# 94} ( 16, 56,108,198,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {# 95} (  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,255,  0,  0),
 {# 96} ( 48, 48, 24,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {# 97} (  0,  0,  0,  0,  0,120, 12,124,204,204,204,118,  0,  0,  0,  0),
 {# 98} (  0,  0,224, 96, 96,120,108,102,102,102,102,124,  0,  0,  0,  0),
 {# 99} (  0,  0,  0,  0,  0,124,198,192,192,192,198,124,  0,  0,  0,  0),
 {#100} (  0,  0, 28, 12, 12, 60,108,204,204,204,204,118,  0,  0,  0,  0),
 {#101} (  0,  0,  0,  0,  0,124,198,254,192,192,198,124,  0,  0,  0,  0),
 {#102} (  0,  0, 56,108,100, 96,240, 96, 96, 96, 96,240,  0,  0,  0,  0),
 {#103} (  0,  0,  0,  0,  0,118,204,204,204,204,204,124, 12,204,120,  0),
 {#104} (  0,  0,224, 96, 96,108,118,102,102,102,102,230,  0,  0,  0,  0),
 {#105} (  0,  0, 24, 24,  0, 56, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {#106} (  0,  0,  6,  6,  0, 14,  6,  6,  6,  6,  6,  6,102,102, 60,  0),
 {#107} (  0,  0,224, 96, 96,102,108,120,120,108,102,230,  0,  0,  0,  0),
 {#108} (  0,  0, 56, 24, 24, 24, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {#109} (  0,  0,  0,  0,  0,236,254,214,214,214,214,198,  0,  0,  0,  0),
 {#110} (  0,  0,  0,  0,  0,220,102,102,102,102,102,102,  0,  0,  0,  0),
 {#111} (  0,  0,  0,  0,  0,124,198,198,198,198,198,124,  0,  0,  0,  0),
 {#112} (  0,  0,  0,  0,  0,220,102,102,102,102,102,124, 96, 96,240,  0),
 {#113} (  0,  0,  0,  0,  0,118,204,204,204,204,204,124, 12, 12, 30,  0),
 {#114} (  0,  0,  0,  0,  0,220,118,102, 96, 96, 96,240,  0,  0,  0,  0),
 {#115} (  0,  0,  0,  0,  0,124,198, 96, 56, 12,198,124,  0,  0,  0,  0),
 {#116} (  0,  0, 16, 48, 48,252, 48, 48, 48, 48, 54, 28,  0,  0,  0,  0),
 {#117} (  0,  0,  0,  0,  0,204,204,204,204,204,204,118,  0,  0,  0,  0),
 {#118} (  0,  0,  0,  0,  0,102,102,102,102,102, 60, 24,  0,  0,  0,  0),
 {#119} (  0,  0,  0,  0,  0,198,198,214,214,214,254,108,  0,  0,  0,  0),
 {#120} (  0,  0,  0,  0,  0,198,108, 56, 56, 56,108,198,  0,  0,  0,  0),
 {#121} (  0,  0,  0,  0,  0,198,198,198,198,198,198,126,  6, 12,248,  0),
 {#122} (  0,  0,  0,  0,  0,254,204, 24, 48, 96,198,254,  0,  0,  0,  0),
 {#123} (  0,  0, 14, 24, 24, 24,112, 24, 24, 24, 24, 14,  0,  0,  0,  0),
 {#124} (  0,  0, 24, 24, 24, 24,  0, 24, 24, 24, 24, 24,  0,  0,  0,  0),
 {#125} (  0,  0,112, 24, 24, 24, 14, 24, 24, 24, 24,112,  0,  0,  0,  0),
 {#126} (  0,  0,118,220,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#127} (  0,  0,  0,  0, 16, 56,108,198,198,198,254,  0,  0,  0,  0,  0),
 {#128} (  0,  0, 60,102,194,192,192,192,194,102, 60, 12,  6,124,  0,  0),
 {#129} (  0,  0,204,  0,  0,204,204,204,204,204,204,118,  0,  0,  0,  0),
 {#130} (  0, 12, 24, 48,  0,124,198,254,192,192,198,124,  0,  0,  0,  0),
 {#131} (  0, 16, 56,108,  0,120, 12,124,204,204,204,118,  0,  0,  0,  0),
 {#132} (  0,  0,204,  0,  0,120, 12,124,204,204,204,118,  0,  0,  0,  0),
 {#133} (  0, 96, 48, 24,  0,120, 12,124,204,204,204,118,  0,  0,  0,  0),
 {#134} (  0, 56,108, 56,  0,120, 12,124,204,204,204,118,  0,  0,  0,  0),
 {#135} (  0,  0,  0,  0, 60,102, 96, 96,102, 60, 12,  6, 60,  0,  0,  0),
 {#136} (  0, 16, 56,108,  0,124,198,254,192,192,198,124,  0,  0,  0,  0),
 {#137} (  0,  0,198,  0,  0,124,198,254,192,192,198,124,  0,  0,  0,  0),
 {#138} (  0, 96, 48, 24,  0,124,198,254,192,192,198,124,  0,  0,  0,  0),
 {#139} (  0,  0,102,  0,  0, 56, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {#140} (  0, 24, 60,102,  0, 56, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {#141} (  0, 96, 48, 24,  0, 56, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {#142} (  0,198,  0, 16, 56,108,198,198,254,198,198,198,  0,  0,  0,  0),
 {#143} ( 56,108, 56,  0, 56,108,198,198,254,198,198,198,  0,  0,  0,  0),
 {#144} ( 24, 48, 96,  0,254,102, 96,124, 96, 96,102,254,  0,  0,  0,  0),
 {#145} (  0,  0,  0,  0,  0,204,118, 54,126,216,216,110,  0,  0,  0,  0),
 {#146} (  0,  0, 62,108,204,204,254,204,204,204,204,206,  0,  0,  0,  0),
 {#147} (  0, 16, 56,108,  0,124,198,198,198,198,198,124,  0,  0,  0,  0),
 {#148} (  0,  0,198,  0,  0,124,198,198,198,198,198,124,  0,  0,  0,  0),
 {#149} (  0, 96, 48, 24,  0,124,198,198,198,198,198,124,  0,  0,  0,  0),
 {#150} (  0, 48,120,204,  0,204,204,204,204,204,204,118,  0,  0,  0,  0),
 {#151} (  0, 96, 48, 24,  0,204,204,204,204,204,204,118,  0,  0,  0,  0),
 {#152} (  0,  0,198,  0,  0,198,198,198,198,198,198,126,  6, 12,120,  0),
 {#153} (  0,198,  0,124,198,198,198,198,198,198,198,124,  0,  0,  0,  0),
 {#154} (  0,198,  0,198,198,198,198,198,198,198,198,124,  0,  0,  0,  0),
 {#155} (  0, 24, 24, 60,102, 96, 96, 96,102, 60, 24, 24,  0,  0,  0,  0),
 {#156} (  0, 56,108,100, 96,240, 96, 96, 96, 96,230,252,  0,  0,  0,  0),
 {#157} (  0,  0,102,102, 60, 24,126, 24,126, 24, 24, 24,  0,  0,  0,  0),
 {#158} (  0,248,204,204,248,196,204,222,204,204,204,198,  0,  0,  0,  0),
 {#159} (  0, 14, 27, 24, 24, 24,126, 24, 24, 24, 24, 24,216,112,  0,  0),
 {#160} (  0, 24, 48, 96,  0,120, 12,124,204,204,204,118,  0,  0,  0,  0),
 {#161} (  0, 12, 24, 48,  0, 56, 24, 24, 24, 24, 24, 60,  0,  0,  0,  0),
 {#162} (  0, 24, 48, 96,  0,124,198,198,198,198,198,124,  0,  0,  0,  0),
 {#163} (  0, 24, 48, 96,  0,204,204,204,204,204,204,118,  0,  0,  0,  0),
 {#164} (  0,  0,118,220,  0,220,102,102,102,102,102,102,  0,  0,  0,  0),
 {#165} (118,220,  0,198,230,246,254,222,206,198,198,198,  0,  0,  0,  0),
 {#166} (  0, 60,108,108, 62,  0,126,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#167} (  0, 56,108,108, 56,  0,124,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#168} (  0,  0, 48, 48,  0, 48, 48, 96,192,198,198,124,  0,  0,  0,  0),
 {#169} (  0,  0,  0,  0,  0,  0,254,192,192,192,192,  0,  0,  0,  0,  0),
 {#170} (  0,  0,  0,  0,  0,  0,254,  6,  6,  6,  6,  0,  0,  0,  0,  0),
 {#171} (  0,192,192,194,198,204, 24, 48, 96,220,134, 12, 24, 62,  0,  0),
 {#172} (  0,192,192,194,198,204, 24, 48,102,206,158, 62,  6,  6,  0,  0),
 {#173} (  0,  0, 24, 24,  0, 24, 24, 24, 60, 60, 60, 24,  0,  0,  0,  0),
 {#174} (  0,  0,  0,  0,  0, 54,108,216,108, 54,  0,  0,  0,  0,  0,  0),
 {#175} (  0,  0,  0,  0,  0,216,108, 54,108,216,  0,  0,  0,  0,  0,  0),
 {#176} ( 17, 68, 17, 68, 17, 68, 17, 68, 17, 68, 17, 68, 17, 68, 17, 68),
 {#177} ( 85,170, 85,170, 85,170, 85,170, 85,170, 85,170, 85,170, 85,170),
 {#178} (221,119,221,119,221,119,221,119,221,119,221,119,221,119,221,119),
 {#179} ( 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24),
 {#180} ( 24, 24, 24, 24, 24, 24, 24,248, 24, 24, 24, 24, 24, 24, 24, 24),
 {#181} ( 24, 24, 24, 24, 24,248, 24,248, 24, 24, 24, 24, 24, 24, 24, 24),
 {#182} ( 54, 54, 54, 54, 54, 54, 54,246, 54, 54, 54, 54, 54, 54, 54, 54),
 {#183} (  0,  0,  0,  0,  0,  0,  0,254, 54, 54, 54, 54, 54, 54, 54, 54),
 {#184} (  0,  0,  0,  0,  0,248, 24,248, 24, 24, 24, 24, 24, 24, 24, 24),
 {#185} ( 54, 54, 54, 54, 54,246,  6,246, 54, 54, 54, 54, 54, 54, 54, 54),
 {#186} ( 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54, 54),
 {#187} (  0,  0,  0,  0,  0,254,  6,246, 54, 54, 54, 54, 54, 54, 54, 54),
 {#188} ( 54, 54, 54, 54, 54,246,  6,254,  0,  0,  0,  0,  0,  0,  0,  0),
 {#189} ( 54, 54, 54, 54, 54, 54, 54,254,  0,  0,  0,  0,  0,  0,  0,  0),
 {#190} ( 24, 24, 24, 24, 24,248, 24,248,  0,  0,  0,  0,  0,  0,  0,  0),
 {#191} (  0,  0,  0,  0,  0,  0,  0,248, 24, 24, 24, 24, 24, 24, 24, 24),
 {#192} ( 24, 24, 24, 24, 24, 24, 24, 31,  0,  0,  0,  0,  0,  0,  0,  0),
 {#193} ( 24, 24, 24, 24, 24, 24, 24,255,  0,  0,  0,  0,  0,  0,  0,  0),
 {#194} (  0,  0,  0,  0,  0,  0,  0,255, 24, 24, 24, 24, 24, 24, 24, 24),
 {#195} ( 24, 24, 24, 24, 24, 24, 24, 31, 24, 24, 24, 24, 24, 24, 24, 24),
 {#196} (  0,  0,  0,  0,  0,  0,  0,255,  0,  0,  0,  0,  0,  0,  0,  0),
 {#197} ( 24, 24, 24, 24, 24, 24, 24,255, 24, 24, 24, 24, 24, 24, 24, 24),
 {#198} ( 24, 24, 24, 24, 24, 31, 24, 31, 24, 24, 24, 24, 24, 24, 24, 24),
 {#199} ( 54, 54, 54, 54, 54, 54, 54, 55, 54, 54, 54, 54, 54, 54, 54, 54),
 {#200} ( 54, 54, 54, 54, 54, 55, 48, 63,  0,  0,  0,  0,  0,  0,  0,  0),
 {#201} (  0,  0,  0,  0,  0, 63, 48, 55, 54, 54, 54, 54, 54, 54, 54, 54),
 {#202} ( 54, 54, 54, 54, 54,247,  0,255,  0,  0,  0,  0,  0,  0,  0,  0),
 {#203} (  0,  0,  0,  0,  0,255,  0,247, 54, 54, 54, 54, 54, 54, 54, 54),
 {#204} ( 54, 54, 54, 54, 54, 55, 48, 55, 54, 54, 54, 54, 54, 54, 54, 54),
 {#205} (  0,  0,  0,  0,  0,255,  0,255,  0,  0,  0,  0,  0,  0,  0,  0),
 {#206} ( 54, 54, 54, 54, 54,247,  0,247, 54, 54, 54, 54, 54, 54, 54, 54),
 {#207} ( 24, 24, 24, 24, 24,255,  0,255,  0,  0,  0,  0,  0,  0,  0,  0),
 {#208} ( 54, 54, 54, 54, 54, 54, 54,255,  0,  0,  0,  0,  0,  0,  0,  0),
 {#209} (  0,  0,  0,  0,  0,255,  0,255, 24, 24, 24, 24, 24, 24, 24, 24),
 {#210} (  0,  0,  0,  0,  0,  0,  0,255, 54, 54, 54, 54, 54, 54, 54, 54),
 {#211} ( 54, 54, 54, 54, 54, 54, 54, 63,  0,  0,  0,  0,  0,  0,  0,  0),
 {#212} ( 24, 24, 24, 24, 24, 31, 24, 31,  0,  0,  0,  0,  0,  0,  0,  0),
 {#213} (  0,  0,  0,  0,  0, 31, 24, 31, 24, 24, 24, 24, 24, 24, 24, 24),
 {#214} (  0,  0,  0,  0,  0,  0,  0, 63, 54, 54, 54, 54, 54, 54, 54, 54),
 {#215} ( 54, 54, 54, 54, 54, 54, 54,255, 54, 54, 54, 54, 54, 54, 54, 54),
 {#216} ( 24, 24, 24, 24, 24,255, 24,255, 24, 24, 24, 24, 24, 24, 24, 24),
 {#217} ( 24, 24, 24, 24, 24, 24, 24,248,  0,  0,  0,  0,  0,  0,  0,  0),
 {#218} (  0,  0,  0,  0,  0,  0,  0, 31, 24, 24, 24, 24, 24, 24, 24, 24),
 {#219} (255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255),
 {#220} (  0,  0,  0,  0,  0,  0,  0,255,255,255,255,255,255,255,255,255),
 {#221} (240,240,240,240,240,240,240,240,240,240,240,240,240,240,240,240),
 {#222} ( 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15),
 {#223} (255,255,255,255,255,255,255,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#224} (  0,  0,  0,  0,  0,118,220,216,216,216,220,118,  0,  0,  0,  0),
 {#225} (  0,  0,120,204,204,204,216,204,198,198,198,204,  0,  0,  0,  0),
 {#226} (  0,  0,254,198,198,192,192,192,192,192,192,192,  0,  0,  0,  0),
 {#227} (  0,  0,  0,  0,254,108,108,108,108,108,108,108,  0,  0,  0,  0),
 {#228} (  0,  0,  0,254,198, 96, 48, 24, 48, 96,198,254,  0,  0,  0,  0),
 {#229} (  0,  0,  0,  0,  0,126,216,216,216,216,216,112,  0,  0,  0,  0),
 {#230} (  0,  0,  0,  0,102,102,102,102,102,124, 96, 96,192,  0,  0,  0),
 {#231} (  0,  0,  0,  0,118,220, 24, 24, 24, 24, 24, 24,  0,  0,  0,  0),
 {#232} (  0,  0,  0,126, 24, 60,102,102,102, 60, 24,126,  0,  0,  0,  0),
 {#233} (  0,  0,  0, 56,108,198,198,254,198,198,108, 56,  0,  0,  0,  0),
 {#234} (  0,  0, 56,108,198,198,198,108,108,108,108,238,  0,  0,  0,  0),
 {#235} (  0,  0, 30, 48, 24, 12, 62,102,102,102,102, 60,  0,  0,  0,  0),
 {#236} (  0,  0,  0,  0,  0,126,219,219,219,126,  0,  0,  0,  0,  0,  0),
 {#237} (  0,  0,  0,  3,  6,126,219,219,243,126, 96,192,  0,  0,  0,  0),
 {#238} (  0,  0, 28, 48, 96, 96,124, 96, 96, 96, 48, 28,  0,  0,  0,  0),
 {#239} (  0,  0,  0,124,198,198,198,198,198,198,198,198,  0,  0,  0,  0),
 {#240} (  0,  0,  0,  0,254,  0,  0,254,  0,  0,254,  0,  0,  0,  0,  0),
 {#241} (  0,  0,  0,  0, 24, 24,126, 24, 24,  0,  0,255,  0,  0,  0,  0),
 {#242} (  0,  0,  0, 48, 24, 12,  6, 12, 24, 48,  0,126,  0,  0,  0,  0),
 {#243} (  0,  0,  0, 12, 24, 48, 96, 48, 24, 12,  0,126,  0,  0,  0,  0),
 {#244} (  0,  0, 14, 27, 27, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24),
 {#245} ( 24, 24, 24, 24, 24, 24, 24, 24,216,216,216,112,  0,  0,  0,  0),
 {#246} (  0,  0,  0,  0, 24, 24,  0,126,  0, 24, 24,  0,  0,  0,  0,  0),
 {#247} (  0,  0,  0,  0,  0,118,220,  0,118,220,  0,  0,  0,  0,  0,  0),
 {#248} (  0, 56,108,108, 56,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#249} (  0,  0,  0,  0,  0,  0,  0, 24, 24,  0,  0,  0,  0,  0,  0,  0),
 {#250} (  0,  0,  0,  0,  0,  0,  0,  0, 24,  0,  0,  0,  0,  0,  0,  0),
 {#251} (  0, 15, 12, 12, 12, 12, 12,236,108,108, 60, 28,  0,  0,  0,  0),
 {#252} (  0,216,108,108,108,108,108,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#253} (  0,112,216, 48, 96,200,248,  0,  0,  0,  0,  0,  0,  0,  0,  0),
 {#254} (  0,  0,  0,  0,124,124,124,124,124,124,124,  0,  0,  0,  0,  0),
 {#255} (  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0)
 );

DosPalette:TDosPalette=(
 (peRed:  0;peGreen:  0;peBlue:  0;peFlags:PC_NOCOLLAPSE),
 (peRed:  0;peGreen:  0;peBlue:168;peFlags:PC_NOCOLLAPSE),
 (peRed:  0;peGreen:168;peBlue:  0;peFlags:PC_NOCOLLAPSE),
 (peRed:  0;peGreen:168;peBlue:168;peFlags:PC_NOCOLLAPSE),
 (peRed:168;peGreen:  0;peBlue:  0;peFlags:PC_NOCOLLAPSE),
 (peRed:168;peGreen:  0;peBlue:168;peFlags:PC_NOCOLLAPSE),
 (peRed:168;peGreen: 84;peBlue:  0;peFlags:PC_NOCOLLAPSE),
 (peRed:168;peGreen:168;peBlue:168;peFlags:PC_NOCOLLAPSE),
 (peRed: 84;peGreen: 84;peBlue: 84;peFlags:PC_NOCOLLAPSE),
 (peRed: 84;peGreen: 84;peBlue:252;peFlags:PC_NOCOLLAPSE),
 (peRed: 84;peGreen:252;peBlue: 84;peFlags:PC_NOCOLLAPSE),
 (peRed: 84;peGreen:252;peBlue:252;peFlags:PC_NOCOLLAPSE),
 (peRed:252;peGreen: 84;peBlue: 84;peFlags:PC_NOCOLLAPSE),
 (peRed:252;peGreen: 84;peBlue:252;peFlags:PC_NOCOLLAPSE),
 (peRed:252;peGreen:252;peBlue: 84;peFlags:PC_NOCOLLAPSE),
 (peRed:252;peGreen:252;peBlue:252;peFlags:PC_NOCOLLAPSE)
);

Constructor TConsole.Create(AOwner:TComponent);
 var
  i:integer;
 begin
  inherited Create(AOwner);
  fUpdate:=0;
  fCols:=80;
  fRows:=25;
  fFont:=Fnt8x16;
  fFontHeight:=16;
  fMemSize:=2*fCols*fRows;
  GetMem(fMem,fMemSize);
  fWhereX:=0;
  fWhereY:=0;
  fTextAttr:=$07;
  fOEMConvert:=True;
  fScroll:=True;
  fPalette:=CreatePalette16;
  SetPaletteEntries(fPalette,0,15,DosPalette[0]);
  fDibWidth:=9*fCols div 2;
  fDibHeight:=fFontHeight*fRows;
  GetMem(fDib,fDibWidth*fDibHeight);
  With fBitmapInfo do begin
   With bmiHeader do begin
    bisize:=SizeOf(TBitmapInfoHeader);
    biWidth:=2*fDibWidth;
    biHeight:=fDibHeight;
    biplanes:=1;
    bibitcount:=4; { for 16 colors only }
    bicompression:=BI_RGB;
    bisizeimage:=0;
    bixpelspermeter:=0;
    biypelspermeter:=0;
    biclrused:=16;
    biclrimportant:=0;
   end;
   For i:=0 to 15 do bmiColors[i]:=i; { cause we're using a dib_pal_colors }
  end;
  TabStop:=True;
  ParentColor:=False;
  Color:=clBlack;
  DoAutoSize;
 end;

Destructor TConsole.Destroy;
 begin
  DeleteObject(fPalette);
  FreeMem(fMem);
  FreeMem(fDib);
  Inherited Destroy;
 end;

Procedure TConsole.Loaded;
 begin
  inherited Loaded;
  ClrScr; // cause Canvas ins't available in the Constructor
 end;

procedure TConsole.WMSetFocus(var Message: TWMSetFocus);
 begin
  inherited;
  if not (csDesigning in ComponentState) then begin
   CreateCaret(Handle,0,9,fFontHeight);
   SetCaretPos(9*fWhereX,fFontHeight*fWhereY);
   ShowCaret(Handle);
  end;
 end;

procedure TConsole.WMKillFocus(Var Message:TWMKillFocus);
 begin
  DestroyCaret;
  inherited;
 end;

procedure TConsole.WMEraseBkGnd(Var Msg:TMessage);
 begin
  Msg.Result:=1;
 end;

Procedure TConsole.SetCols(Value:integer);
 Var
  m:pchar;
  y:integer;
  c:integer;
  o1,o2:integer;
 begin
  if Value<=0 then exit;
  fMemSize:=2*Value*fRows;
  m:=AllocMem(fMemSize);
  if Value<fCols then c:=Value else c:=fCols;
  o1:=0;
  o2:=0;
  for y:=0 to fRows-1 do begin
   Move(fMem[o1],m[o2],c);
   inc(o1,fCols);
   inc(o2,Value);
  end;
  FreeMem(fMem);
  fMem:=m;
  fCols:=Value;
  if WhereX>fCols then GotoXY(fCols,WhereY);
  if fUpdate=0 then NewDib;
 end;

Procedure TConsole.SetRows(Value:integer);
 Var
  m:pchar;
  r:integer;
 begin
  if Value<=0 then exit;
  fMemSize:=2*fCols*Value;
  m:=AllocMem(fMemSize);
  if Value<fRows then r:=Value else r:=fRows;
  Move(fMem^,m^,2*fCols*r);
  FreeMem(fMem);
  fMem:=m;
  fRows:=Value;
  if WhereY>fRows then GotoXY(WhereX,fRows);
  if fUpdate=0 then NewDib;
 end;
(*
Procedure TConsole.SetAutoSize(Value:boolean);
 begin
  if Value<>fAutoSize then begin
   fAutoSize:=Value;
   if fAutoSize then DoAutoSize;
  end;
 end;
*)
Function TConsole.CreatePalette16:HPalette;
 var
  Pal16:pLogPalette;
 begin
  GetMem(Pal16,4+16*4);
  Try
   Pal16^.palVersion:=$300;
   Pal16^.palNumEntries:=16;
   Move(DosPalette,Pal16^.palPalEntry,4*16);
   Result:=CreatePalette(Pal16^);
  Finally
   FreeMem(Pal16,4+16*4);
  end;
 end;

Procedure TConsole.DoAutoSize;
 begin
  Width:=9*fCols;
  Height:=fFontHeight*fRows;
 end;

Procedure TConsole.NewDib;
 var
  x,y,o:integer;
  c:word;
 begin
  FreeMem(fDib);
  with fBitmapInfo.bmiHeader do begin
   biWidth:=9*fCols;
   biHeight:=fDibHeight;
   fDibWidth:=(biWidth+1) div 2;
  end;
  fDibWidth:=(fDibWidth+3)and $FFFC; // DIB Alignment
  fDibHeight:=fFontHeight*fRows;
  GetMem(fDib,fDibWidth*fDibHeight);

  o:=0;
  for y:=0 to fRows-1 do
   for x:=0 to fCols-1 do begin
    c:=MemW[o];
    Putchar(9*x,fFontHeight*y,Chr(c),Hi(c));
    inc(o,2);
   end;

  DoAutoSize;
 end;

procedure TConsole.PutChar(x,y:integer; c:char; color:byte);
 var
  ix,iy:integer;
  b:byte;
  cl:byte;
  r:TRect;
 begin
  for iy:=0 to fFontHeight-1 do begin
   if Font=Fnt8x8 then
    b:=rom8x8[c,iy]
   else
    b:=rom8x16[c,iy];
   for ix:=0 to 7 do begin
    if b>127 then cl:=color and $F else cl:=color shr 4;
    putpixel(x+ix,y+iy,cl);
    b:=byte(b shl 1);
   end;
   putpixel(x+8,y+iy,cl);
  end;
  r:=Rect(x,y,x+9,y+fFontHeight);
  InvalidateRect(Handle,@r,false);
 end;

procedure TConsole.SetChar(x,y:integer; c:char; color:byte);
 var
  i:integer;
 begin
 // update fMem
  i:=2*(x+fCols*y);
  if MemW[i]=color shl 8 +ord(c) then exit;
  fMem[i]:=c;
  fMem[i+1]:=char(color);
 // udate fDib
  if fUpdate=0 then PutChar(9*x,fFontHeight*y,c,color);
 end;


procedure TConsole.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
 begin
  inherited;
  if Focused then begin
   GotoXY(X div 9+1,Y div fFontHeight+1);
   TextAttr:=Mem[2*(fWhereX+fCols*fWhereY)+1];
  end else
   SetFocus;
 end;

procedure TConsole.WMGetDlgCode(Var Msg:TWMGetDlgCode);
 begin
  inherited;
  Msg.Result:=Msg.Result or dlgc_WantArrows or dlgc_WantTab;
 end;

procedure TConsole.KeyDown(var Key: Word; Shift: TShiftState);
 begin
  case Key of
   vk_up   : GotoXY(WhereX,WhereY-1);
   vk_down : if fScroll then MoveTo(WhereX,WhereY+1) else GotoXY(WhereX,WhereY+1);
   vk_left : if WhereX=1 then GotoXY(fCols,WhereY-1) else GotoXY(WhereX-1,WhereY);
   vk_right: MoveTo(WhereX+1,WhereY);
  end;
  inherited;
 end;

procedure TConsole.KeyPress(var Key: Char);
 begin
  Write(Key);
  if Key=#13 then Write(#10);
  inherited;
 end;

Procedure TConsole.ScrollUp;
 var
  x:integer;
  r:TRect;
 begin
  Move(fMem[2*fCols],fMem^,2*fCols*Pred(fRows));
  Move(fDib^,fDib[fFontHeight*fDibWidth],fFontHeight*fDibWidth*Pred(fRows));
  for x:=0 to fCols-1 do SetChar(x,fRows-1,' ',fTextAttr);
  //Invalidate;
  ScrollWindowEx(Handle,0,-fFontHeight,nil,nil,0,@R,0);
  InvalidateRect(Handle,@R,False);
 end;

Procedure TConsole.MoveTo(x,y:integer);
 begin
  if X>fCols then begin
   inc(Y);
   x:=1;
  end;
  if Y>fRows then begin
   ScrollUp;
   dec(y);
  end;
  GotoXY(x,y);
 end;

Function TConsole.GetMemB(Index:integer):byte;
 begin
  if (Index>=0)and(Index<fMemSize) then Result:=ord(fMem[Index]) else Result:=0;
 end;

Procedure TConsole.SetMemB(Index:integer;Value:byte);
 var
  x,y:integer;
 begin
  if (Index>=0)and(Index<fMemSize) then begin
   x:=((Index shr 1) mod fCols);
   y:=((Index shr 1) div fCols);
   if odd(Index) then
    SetChar(x,y,Chr(Mem[Index]),Value)
   else
    SetChar(x,y,Chr(Value),Mem[Index+1]);
  end;
 end;

Function TConsole.GetMemW(Index:integer):word;
 begin
  if (Index>=0)and(Index<fMemSize-1) then begin
   Result:=word(pointer(longint(fMem)+index)^)
  end else
   Result:=0;
 end;

Procedure TConsole.SetMemW(Index:integer;Value:word);
 var
  x,y:integer;
 begin
  if (Index>=0)and(Index<fMemSize) then begin
   x:=((Index shr 1) mod fCols);
   y:=((Index shr 1) div fCols);
   if odd(Index) then begin
    Mem[Index]:=Lo(Value);
    Mem[Index+1]:=Hi(Value);
   end else
    SetChar(x,y,chr(Value),Hi(Value));
  end;
 {
  if (Index>=0)and(Index<fMemSize) then begin
   word(pointer(longint(fMem)+index)^):=Value;
   x:=9*(Index mod (2*fCols));
   y:=fFontHeight*(Index div (2*fCols));
   r:=Rect(x,y,x+18,y+fFontHeight);
   InvalidateRect(Handle,@r,False);
  end;
 }
 end;

Procedure TConsole.SetFont(Value:TRomFont);
 begin
  if fFont<>Value then begin
   fFont:=Value;
   case fFont of
    Fnt8x8 :fFontHeight:=8;
    Fnt8x16:fFontHeight:=16;
   end;
   if fUpdate=0 then NewDib;
  end;
 end;

Procedure TConsole.BeginUpdate;
 begin
  inc(fUpdate);
 end;

Procedure TConsole.EndUpdate;
 begin
  if fUpdate>0 then dec(fUpdate);
  if fUpdate=0 then NewDib;
  Invalidate;
 end;

Procedure TConsole.ClrScr;
 var
  x:integer;
 begin
  For x:=0 to (fCols*fRows)-1 do MemW[2*x]:=fTextAttr shl 8+32;
  GotoXY(1,1);
 end;

Procedure TConsole.ClrEOL;
 var
  o:word;
  x:integer;
 begin
  o:=2*(fWhereX+fCols*fWhereY);
  For x:=WhereX to fCols-1 do begin
   MemW[o]:=fTextAttr shl 8+32;
   inc(o,2);
  end;
 end;

Procedure TConsole.TextMode(Mode:integer);
 begin
  BeginUpdate;
  case Mode of
   CO40:begin Font:=Fnt8x16; Cols:=40; Rows:=25 end;
   CO80:begin Font:=Fnt8x16; Cols:=80; Rows:=25 end;
   CO80+Font8x8: begin Font:=Fnt8x8; Cols:=80; Rows:=50 end;
  end;
  EndUpdate;
 end;

Function TConsole.WhereX:integer;
 begin
  Result:=fWhereX+1;
 end;

Function TConsole.WhereY:integer;
 begin
  Result:=fWhereY+1;
 end;

Procedure TConsole.GotoXY(X,Y:integer);
 begin
  if (X>0)and(X<=fCols)
  and(Y>0)and(Y<=fRows) then begin
   fWhereX:=X-1;
   fWhereY:=Y-1;
   If Focused then SetCaretPos(9*fWhereX,fFontHeight*fWhereY);
  end;
 end;

Procedure TConsole.Write(s:string);
 var
  x:integer;
  y:integer;
 begin
  if fWherex+Length(s)>fCols then begin
   x:=fCols-fWhereX;
   Write(Copy(s,1,x));
   Delete(s,1,x);
   Write(s);
  end else begin
   if OEMConvert then CharToOem(pchar(s),pchar(s));
   for x:=1 to length(s) do begin
    case s[x] of
     #08 : begin MoveTo(WhereX-1,WhereY); SetChar(fWhereX,fWhereY,' ',fTextAttr); end;
     #13 : MoveTo(WhereX,WhereY+1);
     #10 : GotoXY(1,WhereY);
     else begin
      SetChar(fWhereX,fWhereY,s[x],fTextAttr);
      MoveTo(WhereX+1,WhereY);
     end;
    end;
   end;
  end;
 end;

Procedure TConsole.WriteLn(s:string);
 begin
  write(s);
  write(#13#10);
 end;

Procedure TConsole.PutPixel(X,Y,c:integer);
 var
  b:^byte;
 begin
  if (x>=0)and(x<2*fDibWidth)and(y>=0)and(y<fDibHeight) then begin
   b:=@fDib[x div 2+(fDibHeight-y-1)*fDibWidth];
   if odd(x) then
    b^:=b^ and $F0+c
   else
    b^:=b^ and $0F+c shl 4;
  end;
 end;

Procedure TConsole.TextColor(color:byte);
 begin
  fTextAttr:=fTextAttr and $F0 + color;
 end;

Procedure TConsole.TextBackGround(Color:byte);
 begin
  fTextAttr:=fTextAttr and $0F + color shl 4;
 end;

procedure TConsole.Line(x1,y1,x2,y2,color:integer);
 var
  dx,dy,iy:integer;
  f1,f2,f3:integer;
  i:integer;
  r:TRect;
 begin
  dx:=x2-x1;
  dy:=y2-y1;
  if dx<0 then begin
   dx:=-dx;
   dy:=-dy;
   x1:=x2;
   y1:=y2;
  end;
  if dy<0 then begin
   r:=Rect(x1,y1+dy,x1+dx,y1);
   dy:=-dy; iy:=-1;
  end else begin
   r:=Rect(x1,y1,x1+dx,y1+dy);
   iy:=+1;
  end;
  if dx>dy then begin
   f1:=2*dy;
   f2:=f1-dx;
   f3:=f2-dx;
   for i:=0 to dx do begin
    PutPixel(x1,y1,color);
    inc(x1);
    if f2<f1 then inc(f2,f1) else begin
     inc(f2,f3);
     inc(y1,iy);
    end;
   end;
  end else begin
   f1:=2*dx;
   f2:=f1-dy;
   f3:=f2-dy;
   for i:=0 to dy do begin
    PutPixel(x1,y1,color);
    inc(y1,iy);
    if f2<f1 then inc(f2,f1) else begin
     inc(f2,f3);
     inc(x1);
    end;
   end;
  end;
  InvalidateRect(Handle,@r,false);
 end;

procedure TConsole.Ellipse(x1,y1,x2,y2,color:integer);
 var
  r1,r2,cx,cy:integer;
  a:integer;
  lx,ly:integer;
  ax,ay:integer;
  r:TRect;
 begin
  r1:=(x2-x1) div 2; cx:=x1+r1;
  r2:=(y2-y1) div 2; cy:=y1+r2;
  lx:=r1;
  ly:=0;
  for a:=1 to 8 do begin { 90 }
   ax:=round(r1*cos(a*PI/16));
   ay:=round(r2*sin(a*PI/16));
   line(cx+lx,cy+ly,cx+ax,cy+ay,color);
   line(cx-lx,cy+ly,cx-ax,cy+ay,color);
   line(cx+lx,cy-ly,cx+ax,cy-ay,color);
   line(cx-lx,cy-ly,cx-ax,cy-ay,color);
   lx:=ax;
   ly:=ay;
  end;
  r:=Rect(x1,y1,x2,y2);
  InvalidateRect(Handle,@r,false);
 end;

Procedure TConsole.Paint;
 var
  i:integer;
  BitInfo:^TBitmapInfo;
  p:HPalette;
 begin
  BitInfo:=@fBitmapInfo;

  p:=SelectPalette(Canvas.Handle,fPalette,False);
  RealizePalette(Canvas.Handle);
(*
  StretchDIBits(
   Canvas.Handle, // handle of device context
   0,	// x-coordinate of upper-left corner of dest. rect.
   0,	// y-coordinate of upper-left corner of dest. rect.
   Width,	// width of destination rectangle
   Height,	// height of destination rectangle
   0,	// x-coordinate of upper-left corner of source rect.
   0,	// y-coordinate of upper-left corner of source rect.
   2*fDibWidth,	// width of source rectangle
   fDibHeight,	// height of source rectangle
   fDIB,	// address of bitmap bits
   BitInfo^,	// address of bitmap data
   DIB_PAL_COLORS,	// usage
   SRCCOPY 	// raster operation code
  );
*)
  SetDiBitsToDevice(
    Canvas.Handle,	// handle of device context
    0,	// x-coordinate of upper-left corner of dest. rect.
    0,	// y-coordinate of upper-left corner of dest. rect.
    Width,	// source rectangle width
    Height,	// source rectangle height
    0,	// x-coordinate of lower-left corner of source rect.
    0,	// y-coordinate of lower-left corner of source rect.
    0,	// first scan line in array
    Height,	// number of scan lines
    fDIB,	// address of array with DIB bits
    BitInfo^,	// address of structure with bitmap info.
    DIB_PAL_COLORS 	// RGB or palette indices
   );

  SelectPalette(Canvas.Handle,p,False);
 end;

end.
